# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from __future__ import print_function

import codecs
import collections
import inspect
import itertools
import locale
import os
import pipes
import re
import signal
import subprocess as ipc
import sys

import nose
from nose import SkipTest
from nose.tools import (
    assert_equal,
    assert_not_equal,
    assert_true,
)

b''  # Python >= 2.6 is required
if sys.version_info >= (3, 0):
    raise RuntimeError('Python 2.X is required')

re_type = type(re.compile(''))

def noseimport(vmaj, vmin, name=None):
    def wrapper(f):
        if sys.version_info >= (vmaj, vmin):
            return getattr(nose.tools, name or f.__name__)
        return f
    return wrapper

@noseimport(2, 7)
def assert_greater(x, y):
    assert_true(
        x > y,
        msg='{0!r} not greater than {1!r}'.format(x, y)
    )

@noseimport(2, 7)
def assert_in(x, c):
    assert_true(
        x in c,
        msg='{0!r} not found in {1!r}'.format(x, c)
    )

@noseimport(2, 7)
def assert_is(x, y):
    assert_true(
        x is y,
        msg='{0!r} is not {1!r}'.format(x, y)
    )

@noseimport(2, 7)
def assert_is_not(x, y):
    assert_true(
        x is not y,
        msg='{0!r} is {1!r}'.format(x, y)
    )

@noseimport(2, 7)
def assert_is_none(obj):
    assert_is(obj, None)

@noseimport(2, 7)
def assert_is_not_none(obj):
    assert_is_not(obj, None)

@noseimport(2, 7)
def assert_multi_line_equal(x, y):
    return assert_equal(x, y)
if sys.version_info >= (2, 7):
    type(assert_multi_line_equal.__self__).maxDiff = None

@noseimport(2, 7, 'assert_regexp_matches')
def assert_regex(text, regex):
    if isinstance(regex, basestring):
        regex = re.compile(regex)
    if not regex.search(text):
        message = "Regex didn't match: {0!r} not found in {1!r}".format(regex.pattern, text)
        assert_true(False, msg=message)

def _get_signal_names():
    signame_pattern = re.compile('^SIG[A-Z0-9]*$')
    data = dict(
        (name, getattr(signal, name))
        for name in dir(signal)
        if signame_pattern.match(name)
    )
    try:
        if data['SIGABRT'] == data['SIGIOT']:
            del data['SIGIOT']
    except KeyError:
        pass
    try:
        if data['SIGCHLD'] == data['SIGCLD']:
            del data['SIGCLD']
    except KeyError:
        pass
    return dict((no, name) for name, no in data.iteritems())

class _ipc_rc(int):

    _signal_names = _get_signal_names()

    def __repr__(self):
        try:
            return '-' + self._signal_names[-self]
        except KeyError:
            return str(self)

class ipc_result(object):

    def __init__(self, stdout, stderr, rc):
        self.stdout = stdout
        self.stderr = stderr
        self.rc = rc

    def assert_(self, stdout='', stderr='', rc=0):
        if stderr is None:
            pass
        elif isinstance(stderr, re_type):
            assert_regex(self.stderr, stderr)
        else:
            assert_multi_line_equal(self.stderr, stderr)
        if rc is not None:
            assert_equal(_ipc_rc(self.rc), _ipc_rc(rc))
        if stdout is None:
            pass
        elif isinstance(stdout, re_type):
            assert_regex(self.stdout, stdout)
        else:
            assert_multi_line_equal(self.stdout, stdout)

def _get_locale_for_encoding(encoding):
    encoding = codecs.lookup(encoding).name
    candidates = {
        'utf-8': ['C.UTF-8', 'en_US.UTF-8'],
        'iso8859-1': ['en_US.ISO8859-1'],
    }[encoding]
    old_locale = locale.setlocale(locale.LC_ALL)
    try:
        for new_locale in candidates:
            try:
                locale.setlocale(locale.LC_ALL, new_locale)
            except locale.Error:
                continue
            locale_encoding = locale.getpreferredencoding(False)
            locale_encoding = codecs.lookup(locale_encoding).name
            if encoding == locale_encoding:
                return new_locale
    finally:
        locale.setlocale(locale.LC_ALL, old_locale)
    raise SkipTest(
        'locale {loc} is required'.format(loc=' or '.join(candidates))
    )

class case(object):

    _pdf2djvu_command = os.getenv('pdf2djvu') or 'pdf2djvu'
    _feature_cache = {}
    _poppler_version = None

    def get_pdf2djvu_command(self):
        if re.compile(r'\A[[a-zA-Z0-9_+/=.,:%-]+\Z').match(self._pdf2djvu_command):
            return (self._pdf2djvu_command,)
        return ('sh', '-c', self._pdf2djvu_command + ' "$@"', 'sh')

    def get_source_path(self, strip_py=False):
        result = inspect.getsourcefile(type(self))
        if strip_py and result.endswith('.py'):
            return result[:-3]
        else:
            return result

    def get_pdf_path(self):
        return self.get_source_path(strip_py=True) + '.pdf'

    def get_djvu_path(self):
        return self.get_source_path(strip_py=True) + '.djvu'

    def run(self, *commandline, **kwargs):
        env = dict(os.environ,
            MALLOC_CHECK_='3',
            MALLOC_PERTURB_=str(0xA5),
        )
        for key, value in kwargs.items():
            if key.isupper():
                env[key] = value
                continue
            if key == 'encoding':
                env['LC_ALL'] = _get_locale_for_encoding(value)
                continue
            raise TypeError('{key!r} is an invalid keyword argument for this function'.format(key=key))
        print('$', ' '.join(map(pipes.quote, commandline)))
        try:
            child = ipc.Popen(list(commandline),
                stdout=ipc.PIPE,
                stderr=ipc.PIPE,
                env=env,
            )
        except OSError as exc:
            exc.filename = commandline[0]
            raise
        stdout, stderr = child.communicate()
        return ipc_result(stdout, stderr, child.returncode)

    def _pdf2djvu(self, *args, **kwargs):
        quiet = ('-q',) if kwargs.pop('quiet', True) else ()
        args = self.get_pdf2djvu_command() + quiet + (self.get_pdf_path(),) + args
        result = self.run(*args, **kwargs)
        if os.getenv('pdf2djvu_win32'):
            result.stderr = result.stderr.replace('\r\n', '\n')
        if sys.platform.startswith('openbsd'):
            # FIXME: https://github.com/jwilk/pdf2djvu/issues/108
            result.stderr = re.compile(
                r'Magick: Failed to close module [(]"\w*: Invalid handle\"[)].\n'
            ).sub('', result.stderr)
        return result

    def pdf2djvu(self, *args, **kwargs):
        return self._pdf2djvu('-o', self.get_djvu_path(), *args, **kwargs)

    def pdf2djvu_indirect(self, *args):
        return self._pdf2djvu('-i', self.get_djvu_path(), *args)

    def djvudump(self, *args):
        return self.run('djvudump', self.get_djvu_path(), *args)

    def djvused(self, expr, **kwargs):
        return self.run(
            'djvused',
            '-e', expr,
            self.get_djvu_path(),
            **kwargs
        )

    def print_text(self):
        return self.run('djvutxt', self.get_djvu_path())

    def print_outline(self):
        return self.djvused('print-outline')

    def print_ant(self, page):
        return self.djvused('select {0}; print-ant'.format(page))

    def print_meta(self):
        return self.djvused('print-meta')

    def ls(self):
        return self.djvused('ls', encoding='UTF-8')

    def decode(self, mode=None):
        args = []
        if mode is not None:
            args += ['-mode={m}'.format(m=mode)]
        return self.run(
            'ddjvu',
            self.get_djvu_path(),
            '-format=ppm',
            '-subsample=1',
            *args
        )

    def extract_xmp(self):
        r = self.djvused('output-ant')
        assert_equal(r.stderr, '')
        assert_equal(r.rc, 0)
        xmp_lines = [line for line in r.stdout.splitlines() if line.startswith('(xmp "')]
        if len(xmp_lines) == 0:
            return
        [xmp_line] = xmp_lines
        assert_equal(xmp_line[-2:], '")')
        xmp = xmp_line[5:-1]
        xmp = eval(xmp)
        return xmp

    def require_poppler(self, *version):
        if self._poppler_version is None:
            r = self.pdf2djvu('--version')
            r.assert_(stderr=re.compile('^pdf2djvu '), rc=0)
            print(r.stderr)
            match = re.compile('^[+] Poppler ([0-9.]+)$', re.M).search(r.stderr)
            self._poppler_version = tuple(int(x) for x in match.group(1).split('.'))
        if self._poppler_version < version:
            str_version = '.'.join(str(v) for v in version)
            raise SkipTest('Poppler >= {ver} is required'.format(ver=str_version))

    def require_feature(self, feature):
        try:
            feature_enabled = self._feature_cache[feature]
        except KeyError:
            if feature == 'POSIX':
                feature_enabled = not os.getenv('pdf2djvu_win32')
            else:
                r = self.pdf2djvu('--version')
                r.assert_(stdout=re.compile('^pdf2djvu '), rc=0)
                feature_enabled = feature in r.stdout
            self._feature_cache[feature] = feature_enabled
        if not feature_enabled:
            raise SkipTest(feature + ' support missing')

def rainbow(width, height):
    from PIL import Image
    from PIL import ImageColor
    image = Image.new('RGB', (width, height))
    pixels = image.load()
    for x in xrange(width):
        for y in xrange(height):
            hue = 255 * x // (width - 1)
            luminance = 100 * y // height
            color = ImageColor.getrgb('hsl({hue}, 100%, {lum}%)'.format(hue=hue, lum=luminance))
            pixels[x, y] = color
    return image

def checkboard(width, height):
    from PIL import Image
    image = Image.new('1', (width, height))
    pixels = image.load()
    for x in xrange(width):
        for y in xrange(height):
            color = 0xFF * ((x ^ y) & 1)
            pixels[x, y] = color
    return image

_ppm_re = re.compile(r'P6\s+\d+\s+\d+\s+255\s(.*)\Z', re.DOTALL)
def count_ppm_colors(b):
    match = _ppm_re.match(b)
    assert_is_not_none(match)
    pixels = match.group(1)
    ipixels = iter(pixels)
    result = collections.defaultdict(int)
    for pixel in itertools.izip(ipixels, ipixels, ipixels):
        result[pixel] += 1
    return dict(
        (''.join(key), value)
        for key, value in result.iteritems()
    )

xml_ns = dict(
    dc='http://purl.org/dc/elements/1.1/',
    xmpMM='http://ns.adobe.com/xap/1.0/mm/',
)

def xml_find_text(xml, tag):
    [elem] = xml.findall('.//' + tag, xml_ns)
    return elem.text

_uuid_regex = (
    r'\Aurn:uuid:XXXXXXXX-XXXX-4XXX-[89ab]XXX-XXXXXXXXXXXX\Z'
    .replace('X', '[0-9a-f]')
)

def assert_uuid_urn(uuid):
    return assert_regex(
        uuid,
        _uuid_regex,
    )

__all__ = [
    # nose:
    'assert_equal',
    'assert_not_equal',
    'assert_true',
    'assert_in',
    'assert_is',
    'assert_is_none',
    'assert_is_not_none',
    'assert_multi_line_equal',
    'assert_regex',
    # helper classes:
    'ipc_result',
    'case',
    # image handling:
    'rainbow',
    'checkboard',
    'count_ppm_colors',
    # XMP:
    'assert_uuid_urn',
    'xml_find_text',
    'xml_ns',
]

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):
    # + fixed in 0.4.9 [8af81b21de1d8c43bb7585b5318938cde52e30a2]
    # + fixed in 0.4.11 [670a11dc9680cb4dac088d269f08f16f8ec0da7c]

    text = u'¾'
    text_nfkc = '3⁄4'
    text_no_nfkc = text.encode('UTF-8')

    def test_nfkc(self):
        self.pdf2djvu().assert_()
        r = self.print_text()
        r.assert_(stdout=re.compile('^{s} *$'.format(s=self.text_nfkc), re.M))

    def test_no_nfkc(self):
        self.pdf2djvu('--no-nfkc').assert_()
        r = self.print_text()
        r.assert_(stdout=re.compile('^{s} *$'.format(s=self.text_no_nfkc), re.M))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    assert_equal,
    case,
)

class test(case):
    # Bug: https://github.com/jwilk/pdf2djvu/issues/98
    # + fixed in 0.8 [ec2d2e6f7101f7693dac8d102c0dce1fa879f346]

    def test_overwrite(self):
        pdf_path = self.get_pdf_path()
        with open(pdf_path, 'rb') as pdf_file:
            pdf_before = pdf_file.read()
        cmdline = (self.get_pdf2djvu_command() + (
            '-q',
            self.get_pdf_path(),
            '-o', self.get_pdf_path()
        ))
        r = self.run(*cmdline)
        r.assert_(stderr=re.compile('Input file is the same as output file:'), rc=1)
        with open(pdf_path, 'rb') as pdf_file:
            pdf_after = pdf_file.read()
        assert_equal(pdf_before, pdf_after)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):

    def test_no_title(self):
        def t(*args):
            self.pdf2djvu(*args).assert_()
            r = self.ls()
            r.assert_(stdout=re.compile(
                r'\n'
                r'\s*1\s+P\s+\d+\s+[\w.]+\n'
                r'\s*2\s+P\s+\d+\s+[\w.]+\n'
            ))
        t('--page-title-template', '')
        t('--no-page-titles')

    def test_ascii(self):
        template = '#{page}'
        self.pdf2djvu('--page-title-template', template).assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+[\w.]+\s+T=#1\n'
            r'\s*2\s+P\s+\d+\s+[\w.]+\s+T=#2\n'
        ))

    def test_utf8(self):
        self.require_feature('POSIX')
        template = '№{page}'
        self.pdf2djvu('--page-title-template', template, encoding='UTF-8').assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+[\w.]+\s+T=№1\n'
            r'\s*2\s+P\s+\d+\s+[\w.]+\s+T=№2\n'
        ))

    def test_bad_encoding(self):
        self.require_feature('POSIX')
        template = '{page}\xBA'
        r = self.pdf2djvu('--page-title-template', template, encoding='UTF-8')
        r.assert_(stderr=re.compile('^Unable to convert page title to UTF-8:'), rc=1)

    def test_iso8859_1(self):
        template = '{page}\xBA'
        self.pdf2djvu('--page-title-template', template, encoding='ISO8859-1').assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+[\w.]+\s+T=1º\n'
            r'\s*2\s+P\s+\d+\s+[\w.]+\s+T=2º\n'
        ))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2010-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    assert_equal,
    assert_in,
    case,
    count_ppm_colors,
)

class test(case):

    def test(self):
        # Bug: https://github.com/jwilk/pdf2djvu/issues/47
        # + fixed in 0.7.2 [a10cfc8ac94e8f3b7e01e089056bf6a371d1a68d]
        def t(i, n):
            self.require_feature('GraphicsMagick')
            self.pdf2djvu(
                '--dpi=72',
                '--fg-colors={0}'.format(i)
            ).assert_()
            r = self.decode()
            r.assert_(stdout=None)
            r = self.decode(mode='foreground')
            r.assert_(stdout=None)
            colors = count_ppm_colors(r.stdout)
            if isinstance(n, tuple):
                assert_in(len(colors), n)
            else:
                assert_equal(len(colors), n)
        yield t, 1, 2
        yield t, 2, 3
        yield t, 4, 5
        yield t, 255, 241
        yield t, 256, (245, 256)
        yield t, 652, (245, 325)

    def test_range_error(self):
        def t(i):
            self.require_feature('GraphicsMagick')
            r = self.pdf2djvu('--fg-colors={0}'.format(i))
            r.assert_(
                stderr=re.compile('^The specified number of foreground colors is outside the allowed range: 1 .. 4080\n'),
                rc=1,
            )
        t('-1')
        t(0)
        t(4081)

    def test_bad_number(self):
        def t(i):
            self.require_feature('GraphicsMagick')
            r = self.pdf2djvu('--fg-colors={0}'.format(i))
            r.assert_(
                stderr=re.compile('^"{0}" is not a valid number\n'.format(i)),
                rc=1,
            )
        t('')
        t('1x')
        t('0x1')
        t(23 ** 17)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2016 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    case,
)

expected_outline_sexpr = '''\
(bookmarks
 ("Lorem"
  "#p0001.djvu"
  ("ipsum"
   "#p0002.djvu"
   ("dolor"
    "#p0001.djvu" )
   ("sit"
    "#p0002.djvu" ) )
  ("amet"
   "#p0001.djvu" )
  ("consectetur adipisci"
   "#p0002.djvu" ) )
 ("velit"
  "#p0001.djvu" ) )
'''

class test(case):

    def test_multi_page(self):
        self.pdf2djvu().assert_()
        self.print_outline().assert_(stdout=expected_outline_sexpr)

    def test_single_page(self):
        # Make sure that outline is preserved in single-page documents without
        # shared annotation chunk.
        self.pdf2djvu('-p1', '--no-metadata').assert_()
        self.print_outline().assert_(stdout=expected_outline_sexpr)

    def test_iff_corruption(self):
        # Make sure that the NAVM chunk begins on an even byte.
        #
        # Bug: https://github.com/jwilk/pdf2djvu/issues/110
        # + introduced in 0.7.20
        # + fixed in 0.8.2 [cef7b917bf1cde70884d444a4187fcebeec10ba8]
        #
        # This particular choice of options seems to trigger odd-sized DIRM chunk:
        self.pdf2djvu('-p1', '--no-metadata', '--page-title-template', 'xx').assert_()
        self.print_outline().assert_(stdout=expected_outline_sexpr)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2016 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    case,
)

b = 999
item_template = '''
  ("ipsum {0}"
   "#p0001.djvu" )
'''.strip('\n')
expected_outline_sexpr = ('''\
(bookmarks
 ("Lorem"
  "#p0001.djvu"
'''
+ '\n'.join(item_template.format(i) for i in range(1, b + 1))
+ ' ) )\n'
)

class test(case):

    def test_multi_page(self):
        self.pdf2djvu().assert_()
        self.print_outline().assert_(stdout=expected_outline_sexpr)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    case,
)

import re

class test(case):
    def test(self):
        self.pdf2djvu('--guess-dpi').assert_()
        r = self.djvudump()
        r.assert_(stdout=re.compile('INFO .* DjVu 100x200,'))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re
import unicodedata

from tools import (
    case,
)

class test(case):
    # Bug: https://github.com/jwilk/pdf2djvu/issues/90
    # + fixed in 0.8

    text = u'\N{LATIN SMALL LIGATURE FL}uorogra\N{LATIN SMALL LIGATURE FI}a'
    text_nfkc = unicodedata.normalize('NFKC', text).encode('UTF-8')
    text_no_nfkc = text.encode('UTF-8')

    def test_nfkc(self):
        self.pdf2djvu().assert_()
        r = self.print_text()
        r.assert_(stdout=re.compile('^{s} *$'.format(s=self.text_nfkc), re.M))

    def test_no_nfkc(self):
        self.pdf2djvu('--no-nfkc').assert_()
        r = self.print_text()
        r.assert_(stdout=re.compile('^{s} *$'.format(s=self.text_nfkc), re.M))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2016-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):

    def t(self, page, message, empty_stdout=True):
        r = self.pdf2djvu('-p', str(page), quiet=False)
        kwargs = dict(
            stderr=re.compile('- Warning: {warn}\n'.format(warn=re.escape(message)))
        )
        if not empty_stdout:
            kwargs.update(
                stdout=re.compile('')
            )
        r.assert_(**kwargs)

    def test_no_action(self):
        self.t(1, 'Unable to convert link without an action')

    def test_lookup_error(self):
        self.t(2, 'Cannot find link destination',
            empty_stdout=False  # https://bugs.freedesktop.org/show_bug.cgi?id=81513
        )

    def test_remote_goto_action(self):
        self.t(3, 'Unable to convert link with a remote go-to action')

    def test_named_action(self):
        self.t(4, 'Unable to convert link with a named action')

    def test_launch_action(self):
        self.t(5, 'Unable to convert link with a launch action')

    def test_movie_action(self):
        self.t(6, 'Unable to convert link with a multimedia action')

    def test_sound_action(self):
        self.t(7, 'Unable to convert link with a multimedia action')

    def test_rendition_action(self):
        self.t(8, 'Unable to convert link with a multimedia action')

    def test_js_action(self):
        self.t(9, 'Unable to convert link with a JavaScript action')

    def test_set_ocg_state_action(self):
        self.t(10, 'Unable to convert link with a set-OCG-state action')

    def test_unknown_action(self):
        self.t(11, 'Unknown link action')

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):

    def test_11(self):
        # Bug: https://sourceforge.net/p/djvu/bugs/106/
        # + fixed in 0.4.11 [25f63fdcee01a93df16fcd56ebd7587165f4ee52]
        self.pdf2djvu('--bg-subsample=11', '--dpi=72').assert_()
        r = self.djvudump()
        r.assert_(stdout=re.compile('BG44.* 10x11$', re.M))

    def test_12(self):
        # Bug: https://bugs.debian.org/458211
        #
        # Prior to pdf2djvu 0.5.0, subsample ratio 12 was not allowed.
        # Now we require a fixed version of DjVuLibre.
        self.pdf2djvu('--bg-subsample=12', '--dpi=72').assert_()
        r = self.djvudump()
        r.assert_(stdout=re.compile('BG44.* 9x9$', re.M))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2016 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    case,
    assert_greater,
    count_ppm_colors,
)

class test(case):
    # Bug: https://github.com/jwilk/pdf2djvu/issues/115
    # + introduced in 0.7.1 [a9440040faf697bc11370e8012ba586e88e4dca4]
    # + introduced in 0.7.10 [da7cd2524b329a80581b939037f4d42801f3755d]
    # + fixed in 0.9.3 [93f9032d99b75145a03ce64de9c3ae3e25314541]

    def test(self):
        self.pdf2djvu()
        r = self.decode()
        r.assert_(stdout=None)
        r = self.decode(mode='foreground')
        r.assert_(stdout=None)
        colors = count_ppm_colors(r.stdout)
        assert_greater(colors.get('\xFF\0\0', 0), 5000)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2016 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import xml.etree.cElementTree as etree

from tools import (
    assert_equal,
    assert_is_none,
    assert_not_equal,
    assert_uuid_urn,
    case,
    xml_find_text,
)

class test(case):
    # + fixed in 0.6.0 [79d56ebabd600e44f7aa97634d44d471249e74f2]

    def test_verbatim(self):
        self.pdf2djvu('--verbatim-metadata').assert_()
        xmp = self.extract_xmp()
        assert_is_none(xmp)

    def test_no_verbatim(self):
        self.require_feature('Exiv2')
        self.pdf2djvu().assert_()
        xmp = self.extract_xmp()
        xmp = etree.fromstring(xmp)
        dcformat = xml_find_text(xmp, 'dc:format')
        assert_equal(dcformat, 'image/vnd.djvu')
        instance_id = xml_find_text(xmp, 'xmpMM:InstanceID')
        document_id = xml_find_text(xmp, 'xmpMM:DocumentID')
        assert_not_equal(instance_id, document_id)
        assert_uuid_urn(instance_id)
        assert_uuid_urn(document_id)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):

    # Bug: https://github.com/jwilk/pdf2djvu/issues/109
    # + fixed in 0.9 [39920569549418038f1ffcefe65e3ddf78adacd2]

    def test(self):
        def t(*args):
            self.pdf2djvu(*args).assert_()
            r = self.ls()
            r.assert_(stdout=re.compile(
                r'\n'
                r'\s*1\s+P\s+\d+\s+[\w.]+\s+T=one\n'
                r'\s*2\s+P\s+\d+\s+[\w.]+\s+T=Αʹ\n'
                r'\s*3\s+P\s+\d+\s+[\w.]+\s+T=i\n'
                r'\s*4\s+P\s+\d+\s+[\w.]+\s+T=1\n'
            ))
        t()
        t('--page-title-template', '{label}')

    def test_arithmetic(self):
        def t(offset):
            r = self.pdf2djvu('--page-title-template', '{label' + offset + '}')
            r.assert_(
                stderr='Unable to format field {label}: type error: expected number, not string\n',
                rc=1
            )
        t('+1')
        t('-1')

    def test_auto_width(self):
        r = self.pdf2djvu('--page-title-template', '{label:0*}')
        r.assert_(
            stderr='Unable to format field {label}: unknown maximum width\n',
            rc=1
        )

    def test_page_id(self):
        # {label} can be used in --page-title-template,
        # but not it --page-id-template
        r = self.pdf2djvu('--page-id-template', '{label}')
        r.assert_(
            stderr='Unable to format field {label}: no such variable\n',
            rc=1
        )

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):

    # Bug: https://github.com/jwilk/pdf2djvu/issues/112
    # + fixed in 0.9.2 [74a8915ca7602adfd0e71c7d329de2265eec4532]

    def test(self):
        r = self.pdf2djvu()
        r.assert_(
            stderr=re.compile(''),
            stdout=re.compile(''),  # https://bugs.freedesktop.org/show_bug.cgi?id=81513
        )

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):

    def test_default(self):
        self.pdf2djvu().assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+p0001[.]djvu\s+T=1\n'
            r'\s*2\s+P\s+\d+\s+p0002[.]djvu\s+T=2\n'
        ))
        self.pdf2djvu('--pages', '2').assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+p0002[.]djvu\s+T=2\n'
        ))

    def test_spage(self):
        def t(tmpl):
            self.pdf2djvu('--pages', '2', '--page-id-template', tmpl).assert_()
            r = self.ls()
            r.assert_(stdout=re.compile(
                r'\n'
                r'\s*1\s+P\s+\d+\s+p2[.]djvu\sT=2\n'
            ))
        t('p{page}.djvu')
        t('p{spage}.djvu')

    def test_dpage(self):
        self.pdf2djvu('--pages', '2', '--page-id-template', 'p{dpage}.djvu').assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+p1[.]djvu\s+T=2\n'
        ))

    def test_minus(self):
        self.pdf2djvu('--page-id-template', 'p{page-1}.djvu').assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+p0[.]djvu\s+T=1\n'
            r'\s*2\s+P\s+\d+\s+p1[.]djvu\s+T=2\n'
        ))

    def test_plus(self):
        self.pdf2djvu('--page-id-template', 'p{page+7}.djvu').assert_()
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+p8[.]djvu\s+T=1\n'
            r'\s*2\s+P\s+\d+\s+p9[.]djvu\s+T=2\n'
        ))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2018 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import os
import re
import xml.etree.cElementTree as etree

from tools import (
    assert_equal,
    assert_true,
    case,
)

here = os.path.dirname(__file__)
srcdir = os.path.join(here, os.pardir)

class test(case):

    def __init__(self):
        path = os.path.join(srcdir, 'doc', 'changelog')
        with open(path) as file:
            line = file.readline()
        self.changelog_version = line.split()[1].strip('()')

    def test_manpage(self):
        path = os.path.join(srcdir, 'doc', 'manpage.xml')
        for event, elem in etree.iterparse(path):
            if elem.tag == 'refmiscinfo' and elem.get('class') == 'version':
                assert_equal(elem.text, self.changelog_version)
                break
        else:
            assert_true(False, msg="missing <refmiscinfo class='version'>")

    def test_executable(self):
        r = self.pdf2djvu('--version')
        r.assert_(stdout=re.compile('^pdf2djvu [0-9.]+$', re.M), rc=0)
        exec_version = r.stdout.splitlines()[0]
        _, exec_version = exec_version.split()
        assert_equal(exec_version, self.changelog_version)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    assert_not_equal,
    case,
)

import re

class test(case):
    # Bug: https://github.com/jwilk/pdf2djvu/issues/23
    # + fixed in 0.5.5 [ad11da594a254f203f603fd4105fb05c8b508c64]

    # Bug: https://github.com/jwilk/pdf2djvu/issues/73
    # + fixed in 0.8 [5d5fa0e235497aa7e82ea2ba5fcfd3351ae53562]

    def test_bundled(self):
        r = self.pdf2djvu('--pages=1,1')
        r.assert_(stderr=re.compile('^Duplicate page:', re.M), rc=None)
        assert_not_equal(r.rc, 0)

    def test_indirect(self):
        r = self.pdf2djvu_indirect('--pages=1,1')
        r.assert_(stderr=re.compile('^Duplicate page:', re.M), rc=None)
        assert_not_equal(r.rc, 0)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    case,
)

import re

class test(case):
    # Bug: https://github.com/jwilk/pdf2djvu/issues/20
    # + fixed in 0.5.4 [46b70e11778f984c2db302a1c8a18bc0996a387c]

    def test_no_crop(self):
        self.pdf2djvu().assert_()
        r = self.print_text()
        r.assert_(stdout=re.compile('^Lorem ipsum *\n'))

    def test_crop(self):
        self.pdf2djvu('--crop-text').assert_()
        r = self.print_text()
        r.assert_(stdout=re.compile('^Lorem *\n'))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):
    # + fixed in 0.3.3 [0d4169e1bdffb34ebcef51f29abc744ade9c1556]

    def test(self):
        title = (
            r'\316\232\317\211\316\264\316\271\316\272\316\277\317\200\316\277\316\257\316\267\317\203\316\267 '
            r'\317\207\316\261\317\201\316\261\316\272\317\204\316\256\317\201\317\211\316\275'
        )
        self.pdf2djvu('--dpi=72').assert_()
        r = self.print_meta()
        r.assert_(stdout=re.compile('Title\t"{s}"'.format(s=title.replace('\\', '\\\\'))))

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2009-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import re

from tools import (
    case,
)

class test(case):

    def t(self, page, url, border='(xor)'):
        result = self.print_ant(page=page)
        template = '(maparea "{url}" "" (rect NNN NNN NNN NNN) {border})'.format(
            url=url,
            border=border,
        )
        regex = re.escape(template).replace('NNN', '[0-9]+')
        result.assert_(stdout=re.compile(regex))

    def test(self):
        # Bug: https://github.com/jwilk/pdf2djvu/issues/3
        # + fixed in 0.4.10 [47cde6c195ac057c0061a5fa192c69c3373c14ec]
        # + fixed in 0.4.12 [5e691b2b67b3f50275b12d2d262d38646dacda64]
        self.pdf2djvu().assert_()
        self.t(1, '#p0002.djvu')
        self.t(2, '#p0001.djvu', '(border #ff7f00)')
        self.t(3, 'http://www.example.org/')

    def test_border_avis(self):
        def t(*args):
            self.pdf2djvu(*args).assert_()
            self.t(1, '#p0002.djvu', '(xor) (border_avis)')
            self.t(2, '#p0001.djvu', '(border #ff7f00) (border_avis)')
            self.t(3, 'http://www.example.org/', '(xor) (border_avis)')
        t('--hyperlinks', 'border-avis')
        t('--hyperlinks', 'border_avis')

    def test_border_color(self):
        self.pdf2djvu('--hyperlinks', '#3742ff').assert_()
        self.t(1, '#p0002.djvu', '(border #3742ff)')
        self.t(2, '#p0001.djvu', '(border #3742ff)')
        self.t(3, 'http://www.example.org/', '(border #3742ff)')

    def test_none(self):
        def t(*args):
            self.pdf2djvu(*args).assert_()
            for n in range(3):
                r = self.print_ant(page=(n + 1))
                r.assert_(stdout='')
        t('--hyperlinks', 'none')
        t('--no-hyperlinks')

    def test_bad_argument(self):
        r = self.pdf2djvu('--hyperlinks', 'off')
        r.assert_(stderr=re.compile('^Unable to parse hyperlinks options\n'), rc=1)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2010-2016 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    assert_equal,
    case,
    count_ppm_colors,
)

class test(case):
    # Bug: https://github.com/jwilk/pdf2djvu/issues/45
    # + fixed in 0.7.2 [72be13a3bc33817030125301fa34df3ddef63a10]

    def test(self):
        yield self._test, 'default', 325
        yield self._test, 'web', 43
        yield self._test, 'black', 2

    def _test(self, method, n):
        self.pdf2djvu(
            '--dpi=72',
            '--fg-colors={method}'.format(method=method)
        ).assert_()
        r = self.decode()
        r.assert_(stdout=None)
        r = self.decode(mode='foreground')
        r.assert_(stdout=None)
        colors = count_ppm_colors(r.stdout)
        assert_equal(len(colors), n)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2016-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

import contextlib
import re
import resource
import string
import sys

from tools import (
    SkipTest,
    case,
)

@contextlib.contextmanager
def vm_limit(limit):
    code = 'import resource\nfor n in resource.getrlimit(resource.RLIMIT_AS):\n print(n)'
    try:
        RLIMIT_AS = resource.RLIMIT_AS
    except AttributeError:
        # OpenBSD doesn't have RLIMIT_AS;
        # RLIMIT_DATA is good enough.
        RLIMIT_AS = resource.RLIMIT_DATA
        code = re.sub(r'_AS\b', '_DATA', code)
    [lim_soft, lim_hard] = resource.getrlimit(RLIMIT_AS)
    if lim_hard != resource.RLIM_INFINITY and lim_hard < limit:
        limit = lim_hard
    resource.setrlimit(RLIMIT_AS, (limit, lim_hard))
    try:
        r = case().run(sys.executable, '-c', code)
        r.assert_(stdout=re.compile(''))
        (cld_soft_lim, cld_hard_lim) = map(int, r.stdout.splitlines())
        if cld_soft_lim != limit or cld_hard_lim != lim_hard:
            message = 'virtual memory limit did not propagate to subprocess'
            if sys.platform.rstrip(string.digits) == 'gnu':
                raise SkipTest(message + ': https://savannah.gnu.org/bugs/?43320')
            raise RuntimeError(message)
        yield
    finally:
        resource.setrlimit(RLIMIT_AS, (lim_soft, lim_hard))

class test(case):
    # Bug: https://github.com/jwilk/pdf2djvu/issues/107
    # + fixed in 0.9.4 [fb1dcf60c77dfc39de80d60288449caf767d8b54]

    def test(self):
        with vm_limit(1 << 30):  # 1 GiB virtual memory limit
            r = self.pdf2djvu()
        r.assert_(stderr=re.compile(
            'Out of memory\n'
            '|AddressSanitizer failed to allocate '
        ), rc=1)

# vim:ts=4 sts=4 sw=4 et



# encoding=UTF-8

# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
#
# This file is part of pdf2djvu.
#
# pdf2djvu is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# pdf2djvu is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

from tools import (
    case,
)

import re

class test(case):

    # Bug: https://github.com/jwilk/pdf2djvu/issues/113
    # + fixed in 0.9.5 [1b262b90854cd3d5359cd7fdf6b72642b54c8f60]
    def test(self):
        r = self.pdf2djvu('--page-title-template', '{label}', quiet=False)
        r.assert_(stderr=re.compile('Warning: Ignoring duplicate page title: x\n'))
        r = self.ls()
        r.assert_(stdout=re.compile(
            r'\n'
            r'\s*1\s+P\s+\d+\s+[\w.]+\s+T=x\n'
            r'\s*2\s+P\s+\d+\s+[\w.]+\n'
            r'\s*3\s+P\s+\d+\s+[\w.]+\n'
        ))

# vim:ts=4 sts=4 sw=4 et
